@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
 
 #include <gtk/gtk.h>
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 
 #include "bar.h"
 #include "easytag.h"
@@ -40,6 +40,9 @@
  * Declaration *
  ***************/
 static GtkWidget *StatusBar = NULL;
+static guint StatusBarContext;
+static guint timer_cid;
+static guint tooltip_cid;
 static guint StatusbarTimerId = 0;
 static GList *ActionPairsList = NULL;
 
@@ -52,6 +55,18 @@ static void Check_Menu_Item_Toggled_Browse_Subdir (GtkWidget *checkmenuitem);
 static void Init_Menu_Bar (void);
 static void Statusbar_Remove_Timer (void);
 
+static void et_statusbar_push_tooltip (const gchar *message);
+static void et_statusbar_pop_tooltip (void);
+static void et_ui_manager_on_connect_proxy (GtkUIManager *manager,
+                                            GtkAction *action,
+                                            GtkWidget *proxy,
+                                            gpointer user_data);
+static void et_ui_manager_on_disconnect_proxy (GtkUIManager *manager,
+                                               GtkAction *action,
+                                               GtkWidget *proxy,
+                                               gpointer user_data);
+static void on_menu_item_select (GtkMenuItem *item, gpointer user_data);
+static void on_menu_item_deselect (GtkMenuItem *item, gpointer user_data);
 
 /*************
  * Functions o
@@ -69,7 +84,6 @@ static void Statusbar_Remove_Timer (void);
 static void
 Menu_Sort_Action (GtkAction *item, gpointer data)
 {
-    GtkWidget *TBViewMode;
     const gchar *action = gtk_action_get_name(item);
     GQuark quark = g_quark_from_string(action);
 
@@ -114,21 +128,13 @@ Menu_Sort_Action (GtkAction *item, gpointer data)
     QCASE_DATA(AM_SORT_ASCENDING_FILE_SAMPLERATE,  ET_Sort_Displayed_File_List_And_Update_UI, SORTING_BY_ASCENDING_FILE_SAMPLERATE);
     QCASE_DATA(AM_SORT_DESCENDING_FILE_SAMPLERATE, ET_Sort_Displayed_File_List_And_Update_UI, SORTING_BY_DESCENDING_FILE_SAMPLERATE);
     QCASE_DATA(AM_INITIALIZE_TREE,                 Browser_Tree_Rebuild,                      NULL);
-
-    if (quark == g_quark_from_string(AM_TREE_OR_ARTISTALBUM_VIEW))
-    {
-        // Toggle button to switch between Browser view and Artist / Album view
-        TBViewMode = gtk_ui_manager_get_widget(UIManager, "/ToolBar/ViewModeToggle");
-        gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(TBViewMode),
-                                          !gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(TBViewMode)));
-    }
+    Browser_List_Refresh_Sort ();
 }
 
 void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
 {
     GtkWidget *menubar;
     GtkWidget *toolbar;
-    GtkWidget *toolbarwidget;
 
     /*
      * Structure :
@@ -146,8 +152,10 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
          * Main Menu Actions
          */
         { MENU_FILE,                          NULL,                      _("_File"),                         NULL, NULL,                               NULL},
-        { MENU_FILE_SORT_TAG,                 GTK_STOCK_SORT_ASCENDING,  _("Sort list by tag"),              NULL, NULL,                               NULL },
-        { MENU_FILE_SORT_PROP,                GTK_STOCK_SORT_ASCENDING,  _("Sort list by property"),         NULL, NULL,                               NULL },
+        { MENU_FILE_SORT_TAG, GTK_STOCK_SORT_ASCENDING, _("Sort List by Tag"),
+          NULL, NULL, NULL },
+        { MENU_FILE_SORT_PROP, GTK_STOCK_SORT_ASCENDING,
+          _("Sort List by Property"), NULL, NULL, NULL },
         { AM_SORT_ASCENDING_FILENAME,         GTK_STOCK_SORT_ASCENDING,  _("Ascending by filename"),         NULL, _("Ascending by filename"),         G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_DESCENDING_FILENAME,        GTK_STOCK_SORT_DESCENDING, _("Descending by filename"),        NULL, _("Descending by filename"),        G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_ASCENDING_CREATION_DATE,    GTK_STOCK_SORT_ASCENDING,  _("Ascending by creation date"),    NULL, _("Ascending by creation date"),    G_CALLBACK(Menu_Sort_Action) },
@@ -176,7 +184,11 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
         { AM_SORT_DESCENDING_COPYRIGHT,       GTK_STOCK_SORT_DESCENDING, _("Descending by copyright"),       NULL, _("Descending by copyright"),       G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_ASCENDING_URL,              GTK_STOCK_SORT_ASCENDING,  _("Ascending by URL"),              NULL, _("Ascending by URL"),              G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_DESCENDING_URL,             GTK_STOCK_SORT_DESCENDING, _("Descending by URL"),             NULL, _("Descending by URL"),             G_CALLBACK(Menu_Sort_Action) },
+	/* Translators: the encoder name is supposed to be the name of a person
+         * or organisation, but can sometimes be the name of an application. */
         { AM_SORT_ASCENDING_ENCODED_BY,       GTK_STOCK_SORT_ASCENDING,  _("Ascending by encoder name"),     NULL, _("Ascending by encoder name"),     G_CALLBACK(Menu_Sort_Action) },
+	/* Translators: the encoder name is supposed to be the name of a person
+         * or organisation, but can sometimes be the name of an application. */
         { AM_SORT_DESCENDING_ENCODED_BY,      GTK_STOCK_SORT_DESCENDING, _("Descending by encoder name"),    NULL, _("Descending by encoder name"),    G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_ASCENDING_FILE_TYPE,        GTK_STOCK_SORT_ASCENDING,  _("Ascending by file type"),        NULL, _("Ascending by file type"),        G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_DESCENDING_FILE_TYPE,       GTK_STOCK_SORT_DESCENDING, _("Descending by file type"),       NULL, _("Descending by file type"),       G_CALLBACK(Menu_Sort_Action) },
@@ -189,56 +201,108 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
         { AM_SORT_ASCENDING_FILE_SAMPLERATE,  GTK_STOCK_SORT_ASCENDING,  _("Ascending by samplerate"),       NULL, _("Ascending by samplerate"),       G_CALLBACK(Menu_Sort_Action) },
         { AM_SORT_DESCENDING_FILE_SAMPLERATE, GTK_STOCK_SORT_DESCENDING, _("Descending by samplerate"),      NULL, _("Descending by samplerate"),      G_CALLBACK(Menu_Sort_Action) },
 
-        { AM_OPEN_FILE_WITH,     GTK_STOCK_OPEN,             _("Open Files With…"),      NULL,                _("Open files with…"),     G_CALLBACK(Browser_Open_Run_Program_List_Window) },
-        { AM_SELECT_ALL_FILES,   GTK_STOCK_SELECT_ALL,       _("Select All Files"),           "<Control>A",        _("Select all files"),          G_CALLBACK(Action_Select_All_Files) },
-        { AM_UNSELECT_ALL_FILES, "easytag-unselect-all",     _("Unselect All Files"),         "<Shift><Control>A", _("Unselect all files"),        G_CALLBACK(Action_Unselect_All_Files) },
-        { AM_INVERT_SELECTION,   "easytag-invert-selection", _("Invert Files Selection"),     "<Control>I",        _("Invert files selection"),    G_CALLBACK(Action_Invert_Files_Selection) },
+        { AM_OPEN_FILE_WITH, GTK_STOCK_OPEN, _("Open Files With…"),
+          "<Primary><Shift>O", _("Run a command on the selected files"),
+          G_CALLBACK (Browser_Open_Run_Program_List_Window) },
+        { AM_SELECT_ALL, GTK_STOCK_SELECT_ALL, NULL, "<Primary>A",
+          _("Select all"), G_CALLBACK (et_on_action_select_all) },
+        { AM_UNSELECT_ALL, "easytag-unselect-all", _("Unselect All"),
+          "<Primary><Shift>A", _("Clear the current selection"),
+          G_CALLBACK (et_on_action_unselect_all) },
+        { AM_INVERT_SELECTION, "easytag-invert-selection",
+          _("Invert File Selection"), "<Primary>I",
+          _("Invert file selection"),
+          G_CALLBACK (Action_Invert_Files_Selection) },
         { AM_DELETE_FILE,        GTK_STOCK_DELETE,           _("Delete Files"),             NULL,                _("Delete files"),            G_CALLBACK(Action_Delete_Selected_Files) },
-        { AM_FIRST,              GTK_STOCK_GOTO_FIRST,       _("_First File"),                "<Control>Home",     _("First file"),                G_CALLBACK(Action_Select_First_File) },
+        { AM_FIRST, GTK_STOCK_GOTO_FIRST, _("_First File"), "<Primary>Home",
+          _("First file"), G_CALLBACK (Action_Select_First_File) },
         { AM_PREV,               GTK_STOCK_GO_BACK,          _("_Previous File"),             "Page_Up",           _("Previous file"),             G_CALLBACK(Action_Select_Prev_File) },
         { AM_NEXT,               GTK_STOCK_GO_FORWARD,       _("_Next File"),                 "Page_Down",         _("Next file"),                 G_CALLBACK(Action_Select_Next_File) },
-        { AM_LAST,               GTK_STOCK_GOTO_LAST,        _("_Last File"),                 "<Control>End",      _("Last file"),                 G_CALLBACK(Action_Select_Last_File) },
-        // XXX GTK1 version uses Ctrl+C for scanner, this doesnt work in GTK1 as its copy! in gtk2, behaviour is different
-        // and binding Ctrl+C effectively stops the user copying text..
-        { AM_SCAN,               "document-properties",             _("S_can Files…"),              NULL,                _("Scan files"),              G_CALLBACK(Action_Scan_Selected_Files) },
-        { AM_REMOVE,             GTK_STOCK_CLEAR,            _("_Remove Tags"),             "<Control>R",        _("Remove tags"),             G_CALLBACK(Action_Remove_Selected_Tags) },
-        { AM_UNDO,               GTK_STOCK_UNDO,             _("_Undo Last Files Changes"), "<Control>Z",        _("Undo last Files changes"), G_CALLBACK(Action_Undo_Selected_Files) },
-        { AM_REDO,               GTK_STOCK_REDO,             _("R_edo Last Files Changes"), "<Shift><Control>Z", _("Redo last files changes"), G_CALLBACK(Action_Redo_Selected_File) },
-        { AM_SAVE,               GTK_STOCK_SAVE,             _("_Save Files"),              "<Control>S",        _("Save files"),              G_CALLBACK(Action_Save_Selected_Files) },
-        { AM_SAVE_FORCED,        GTK_STOCK_SAVE,             _("_Force Saving Files"),      "<Shift><Control>S", _("Force saving files"),      G_CALLBACK(Action_Force_Saving_Selected_Files) },
+        { AM_LAST, GTK_STOCK_GOTO_LAST, _("_Last File"), "<Primary>End",
+          _("Last file"), G_CALLBACK (Action_Select_Last_File) },
+        { AM_SCAN_FILES, GTK_STOCK_APPLY, _("S_can Files"), NULL,
+          _("Scan selected files"), G_CALLBACK (Action_Scan_Selected_Files) },
+        { AM_REMOVE, GTK_STOCK_CLEAR, _("_Remove Tags"), "<Primary>E",
+          _("Remove tags"), G_CALLBACK (Action_Remove_Selected_Tags) },
+        { AM_UNDO, GTK_STOCK_UNDO, _("_Undo Last Files Changes"), "<Primary>Z",
+          _("Undo last files changes"),
+          G_CALLBACK(Action_Undo_Selected_Files) },
+        { AM_REDO, GTK_STOCK_REDO, _("R_edo Last Files Changes"),
+          "<Primary><Shift>Z", _("Redo last files changes"),
+          G_CALLBACK (Action_Redo_Selected_File) },
+        { AM_SAVE, GTK_STOCK_SAVE, _("_Save Files"), "<Primary>S",
+          _("Save changes to selected files"),
+          G_CALLBACK(Action_Save_Selected_Files) },
+        { AM_SAVE_FORCED, GTK_STOCK_SAVE, _("_Force Save Files"),
+          "<Primary><Shift>S", _("Force saving files"),
+          G_CALLBACK (Action_Force_Saving_Selected_Files) },
         { AM_UNDO_HISTORY,       GTK_STOCK_UNDO,             _("Undo Last Changes"),          NULL,                _("Undo last changes"),         G_CALLBACK(Action_Undo_From_History_List) },
         { AM_REDO_HISTORY,       GTK_STOCK_REDO,             _("Redo Last Changes"),          NULL,                _("Redo last changes"),         G_CALLBACK(Action_Redo_From_History_List) },
-        { AM_QUIT,               GTK_STOCK_QUIT,             _("_Quit"),                      "<Control>Q",        _("Quit"),                      G_CALLBACK(Quit_MainWindow) },
+        { AM_QUIT, GTK_STOCK_QUIT, _("_Quit"), "<Primary>Q", _("Quit"),
+          G_CALLBACK (Quit_MainWindow) },
 
         { MENU_BROWSER,                NULL,                   _("_Browser"),                      NULL,                NULL,                               NULL },
-        { AM_LOAD_HOME_DIR,            GTK_STOCK_HOME,         _("Go to _Home Directory"),         "<Alt>Home",         _("Go to home directory"),          G_CALLBACK(Browser_Load_Home_Directory) },
-        { AM_LOAD_DESKTOP_DIR,         "user-desktop",      _("Go to Desktop Directory"),       NULL,         		_("Go to desktop directory"),       G_CALLBACK(Browser_Load_Desktop_Directory) },
-        { AM_LOAD_DOCUMENTS_DIR,       "folder-documents",      _("Go to Documents Directory"),     NULL,         		_("Go to documents directory"),     G_CALLBACK(Browser_Load_Documents_Directory) },
-        { AM_LOAD_DOWNLOADS_DIR,       "folder-download",      _("Go to Downloads Directory"),     NULL,         		_("Go to downloads directory"),     G_CALLBACK(Browser_Load_Downloads_Directory) },
-        { AM_LOAD_MUSIC_DIR,           "folder-music",      _("Go to Music Directory"),         NULL,         		_("Go to music directory"),         G_CALLBACK(Browser_Load_Music_Directory) },
-        { AM_LOAD_DEFAULT_DIR,         GTK_STOCK_JUMP_TO,      _("Go to _Default Directory"),      "<Control>D",        _("Go to default directory"),       G_CALLBACK(Browser_Load_Default_Directory) },
+        { AM_LOAD_HOME_DIR, GTK_STOCK_HOME, _("_Home Directory"), "<Alt>Home",
+          _("Go to home directory"),
+          G_CALLBACK (Browser_Load_Home_Directory) },
+        { AM_LOAD_DESKTOP_DIR, "user-desktop", _("Desktop Directory"), NULL,
+          _("Go to desktop directory"),
+          G_CALLBACK (Browser_Load_Desktop_Directory) },
+        { AM_LOAD_DOCUMENTS_DIR, "folder-documents", _("Documents Directory"),
+          NULL, _("Go to documents directory"),
+          G_CALLBACK (Browser_Load_Documents_Directory) },
+        { AM_LOAD_DOWNLOADS_DIR, "folder-download", _("Downloads Directory"),
+          NULL, _("Go to downloads directory"),
+          G_CALLBACK (Browser_Load_Downloads_Directory) },
+        { AM_LOAD_MUSIC_DIR, "folder-music", _("Music Directory"), NULL,
+          _("Go to music directory"),
+          G_CALLBACK (Browser_Load_Music_Directory) },
+        { AM_LOAD_PARENT_DIR, GTK_STOCK_GO_UP, _("_Parent Directory"),
+          "<Alt>Up", _("Go to parent directory"),
+          G_CALLBACK (et_browser_on_action_parent_directory) },
+        { AM_LOAD_DEFAULT_DIR, GTK_STOCK_JUMP_TO, _("_Default Directory"),
+          "<Primary>D", _("Go to default directory"),
+          G_CALLBACK (Browser_Load_Default_Directory) },
         { AM_SET_PATH_AS_DEFAULT,      GTK_STOCK_DIRECTORY,    _("Set _Current Path as Default"),  NULL,                _("Set current path as default"),   G_CALLBACK(Set_Current_Path_As_Default) },
-        { AM_TREE_OR_ARTISTALBUM_VIEW, "easytag-artist-album", _("Tree View | Artist-Album View"), NULL,                _("Toggle between tree view and artist-album view"), G_CALLBACK(Menu_Sort_Action) },
         { AM_RENAME_DIR,               GTK_STOCK_INDEX,        _("Rename Directory…"),          "F2",                _("Rename directory"),          G_CALLBACK(Browser_Open_Rename_Directory_Window) },
-        { AM_RELOAD_DIRECTORY,         GTK_STOCK_REFRESH,      _("Reload Directory"),              "F5",                _("Reload directory"),              G_CALLBACK(Browser_Reload_Directory) },
-        { AM_BROWSE_DIRECTORY_WITH,    GTK_STOCK_EXECUTE,      _("Browse Directory with…"),     NULL,                _("Browse directory with…"),     G_CALLBACK(Browser_Open_Run_Program_Tree_Window) },
-        { AM_COLLAPSE_TREE,            NULL,                   _("_Collapse Tree"),                "<Control><Shift>C", _("_Collapse tree"),                G_CALLBACK(Browser_Tree_Collapse) },
-        { AM_INITIALIZE_TREE,          GTK_STOCK_REFRESH,      _("_Refresh Tree"),                 "<Control><Shift>R", _("_Refresh tree"),                 G_CALLBACK(Browser_Tree_Rebuild) },
+        { AM_RELOAD_DIRECTORY, GTK_STOCK_REFRESH, _("Reload Directory"),
+          "<Primary>R", _("Reload directory"),
+          G_CALLBACK (Browser_Reload_Directory) },
+        { AM_BROWSE_DIRECTORY_WITH, GTK_STOCK_EXECUTE,
+          _("Browse Directory With…"), NULL,
+          _("Run a command on the directory"),
+          G_CALLBACK (Browser_Open_Run_Program_Tree_Window) },
+        { AM_COLLAPSE_TREE, NULL, _("_Collapse Tree"), "<Primary><Shift>C",
+          _("Collapse directory tree"), G_CALLBACK (Browser_Tree_Collapse) },
+        { AM_INITIALIZE_TREE, GTK_STOCK_REFRESH, _("_Reload Tree"),
+          "<Primary><Shift>R", _("Reload directory tree"),
+          G_CALLBACK (Browser_Tree_Rebuild) },
 
-        { MENU_SCANNER,              NULL,                  _("S_canner"),                          NULL,         NULL,                                 NULL },
-        { AM_SCANNER_FILL_TAG,       "document-properties",        _("_Fill Tags…"),                  NULL,         _("Fill tags"),                 G_CALLBACK(Scan_Use_Fill_Tag_Scanner) },
-        { AM_SCANNER_RENAME_FILE,    "document-properties",        _("_Rename Files and Directories…"), NULL,         _("Rename files and directories"),G_CALLBACK(Scan_Use_Rename_File_Scanner) },
-        { AM_SCANNER_PROCESS_FIELDS, "document-properties",        _("_Process Fields…"),             NULL,         _("Process Fields"),           G_CALLBACK(Scan_Use_Process_Fields_Scanner) },
+        { MENU_SCANNER, NULL, _("S_canner Mode"), NULL, NULL, NULL },
 
         { MENU_MISC,                NULL,                   _("_Miscellaneous"),                             NULL,         NULL,                                 NULL },
-        { AM_SEARCH_FILE,           GTK_STOCK_FIND,         _("Find _Files…"),               "<Control>F", _("Find files"),               G_CALLBACK(Open_Search_File_Window) },
-        { AM_CDDB_SEARCH,           GTK_STOCK_CDROM,        _("CD Data_base Search…"),          "<Control>B", _("CDDB search"),                 G_CALLBACK(Open_Cddb_Window) },
-        { AM_FILENAME_FROM_TXT,     GTK_STOCK_OPEN,         _("Load Filenames From Text File…"),       "<Alt>T",     _("Load filenames from text file"),     G_CALLBACK(Open_Load_Filename_Window) },
-        { AM_WRITE_PLAYLIST,        GTK_STOCK_SAVE_AS,      _("Write Playlist…"),                "<Alt>W",     _("Write playlist"),              G_CALLBACK(Open_Write_Playlist_Window) },
-        { AM_RUN_AUDIO_PLAYER,      GTK_STOCK_MEDIA_PLAY,        _("Run Audio Player"),                  "<Alt>X",     _("Run audio player"),                G_CALLBACK(Run_Audio_Player_Using_Selection) },
+        { AM_SEARCH_FILE, GTK_STOCK_FIND, _("_Find…"), "<Primary>F",
+          _("Search filenames and tags"),
+          G_CALLBACK (Open_Search_File_Window) },
+        { AM_CDDB_SEARCH, GTK_STOCK_CDROM, _("CDD_B Search…"), "<Primary>B",
+          _("CDDB search"), G_CALLBACK (Open_Cddb_Window) },
+        { AM_FILENAME_FROM_TXT, GTK_STOCK_OPEN,
+          _("Load Filenames From a Text File…"), "<Primary>T",
+          _("Load filenames from a text file"),
+          G_CALLBACK (Open_Load_Filename_Window) },
+        { AM_WRITE_PLAYLIST, GTK_STOCK_SAVE_AS, _("Generate Playlist…"),
+          "<Primary>W", _("Generate a playlist"),
+          G_CALLBACK (Open_Write_Playlist_Window) },
+        { AM_RUN_AUDIO_PLAYER, GTK_STOCK_MEDIA_PLAY, _("Run Audio Player"),
+          "<Primary>M", _("Run audio player"),
+          G_CALLBACK (Run_Audio_Player_Using_Selection) },
 
-        { MENU_SETTINGS,            NULL,                   _("_Settings"),                         NULL,         NULL,                                 NULL },
-        { AM_OPEN_OPTIONS_WINDOW,   GTK_STOCK_PREFERENCES,  _("_Preferences"),                  "<Alt>P",     _("Preferences"),                 G_CALLBACK(Open_OptionsWindow) },
+        { MENU_EDIT, NULL, _("_Edit"), NULL, NULL, NULL },
+        { AM_OPEN_OPTIONS_WINDOW, GTK_STOCK_PREFERENCES, _("_Preferences"),
+          NULL, _("Preferences"), G_CALLBACK (Open_OptionsWindow) },
+
+        { MENU_VIEW, NULL, _("_View"), NULL, NULL, NULL },
+        { MENU_GO, NULL, _("_Go"), NULL, NULL, NULL },
 
         { MENU_HELP,                NULL,                   _("_Help"),                             NULL,         NULL,                                 NULL },
         { AM_OPEN_ABOUT_WINDOW,     GTK_STOCK_ABOUT,        _("_About"),                            NULL,         _("About"),                           G_CALLBACK(Show_About_Window) },
@@ -275,17 +339,44 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
                     */
         { AM_BROWSER_HIDDEN_DIR, NULL,                   _("Show Hidden Directories"),                         NULL, _("Show hidden directories"),                         G_CALLBACK(Browser_Tree_Rebuild),     BROWSE_HIDDEN_DIR },
 #endif /* !G_OS_WIN32 */
-        { AM_VIEWMODE_TOGGLE,    "easytag-artist-album", _("Show Tree Browser / Display by Artist and Album"), NULL, _("Show tree browser / display by artist and Album"), G_CALLBACK(Action_Select_Browser_Style), FALSE },
+        { AM_SCANNER_SHOW, "document-properties", _("_Show Scanner"), NULL,
+          _("Show scanner"), G_CALLBACK (et_scan_show),
+          OPEN_SCANNER_WINDOW_ON_STARTUP },
+    };
+
+    GtkRadioActionEntry view_mode_entries[] =
+    {
+        { AM_TREE_VIEW_MODE, "audio-x-generic", _("Tree Browser"), NULL,
+          _("View by directory tree"), 0 },
+        { AM_ARTIST_VIEW_MODE, "easytag-artist-album",
+          _("Artist and Album"), NULL,
+          _("View by artist and album"), 1 }
+    };
+
+    GtkRadioActionEntry scanner_mode_entries[] =
+    {
+        { AM_SCANNER_FILL_TAG, "document-properties", _("_Fill Tags…"), NULL,
+          _("Fill tags"), SCANNER_FILL_TAG },
+        { AM_SCANNER_RENAME_FILE, "document-properties",
+          _("_Rename Files and Directories…"), NULL,
+          _("Rename files and directories"), SCANNER_RENAME_FILE },
+        { AM_SCANNER_PROCESS_FIELDS, "document-properties",
+          _("_Process Fields…"), NULL, _("Process Fields"),
+          SCANNER_PROCESS_FIELDS }
     };
 
     GError *error = NULL;
     guint num_menu_entries;
     guint num_toggle_entries;
+    guint n_view_mode_entries;
+    guint n_scanner_mode_entries;
     guint i;
 
     /* Calculate number of items into the menu */
     num_menu_entries = G_N_ELEMENTS(ActionEntries);
     num_toggle_entries = G_N_ELEMENTS(ToggleActionEntries);
+    n_view_mode_entries = G_N_ELEMENTS (view_mode_entries);
+    n_scanner_mode_entries = G_N_ELEMENTS (scanner_mode_entries);
 
     /* Populate quarks list with the entries */
     for(i = 0; i < num_menu_entries; i++)
@@ -293,7 +384,7 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
         Action_Pair* ActionPair = g_malloc0(sizeof(Action_Pair));
         ActionPair->action = ActionEntries[i].name;
         ActionPair->quark  = g_quark_from_string(ActionPair->action);
-        ActionPairsList    = g_list_append(ActionPairsList, ActionPair);
+        ActionPairsList = g_list_prepend (ActionPairsList, ActionPair);
     }
 
     for(i = 0; i < num_toggle_entries; i++)
@@ -301,15 +392,31 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
         Action_Pair* ActionPair = g_malloc0(sizeof(Action_Pair));
         ActionPair->action = ToggleActionEntries[i].name;
         ActionPair->quark  = g_quark_from_string(ActionPair->action);
-        ActionPairsList    = g_list_append(ActionPairsList, ActionPair);
+        ActionPairsList = g_list_prepend (ActionPairsList, ActionPair);
     }
 
+    ActionPairsList = g_list_reverse (ActionPairsList);
+
     /* UI Management */
     ActionGroup = gtk_action_group_new("actions");
+    gtk_action_group_set_translation_domain (ActionGroup, GETTEXT_PACKAGE);
     gtk_action_group_add_actions(ActionGroup, ActionEntries, num_menu_entries, NULL);
     gtk_action_group_add_toggle_actions(ActionGroup, ToggleActionEntries, num_toggle_entries, NULL);
+    gtk_action_group_add_radio_actions (ActionGroup, view_mode_entries,
+                                        n_view_mode_entries, 0,
+                                        Action_Select_Browser_Style, NULL);
+    gtk_action_group_add_radio_actions (ActionGroup, scanner_mode_entries,
+                                        n_scanner_mode_entries, 0,
+                                        G_CALLBACK (et_on_action_select_scan_mode),
+                                        NULL);
 
     UIManager = gtk_ui_manager_new();
+
+    g_signal_connect (UIManager, "connect-proxy",
+                      G_CALLBACK (et_ui_manager_on_connect_proxy), NULL);
+    g_signal_connect (UIManager, "disconnect-proxy",
+                      G_CALLBACK (et_ui_manager_on_disconnect_proxy), NULL);
+
     if (!gtk_ui_manager_add_ui_from_string(UIManager, ui_xml, -1, &error))
     {
         g_error(_("Could not merge UI, error was: %s\n"), error->message);
@@ -318,16 +425,12 @@ void Create_UI (GtkWidget **ppmenubar, GtkWidget **pptoolbar)
     gtk_ui_manager_insert_action_group(UIManager, ActionGroup, 0);
     gtk_window_add_accel_group(GTK_WINDOW(MainWindow), gtk_ui_manager_get_accel_group(UIManager));
 
-    /* Menu (not placed in a handlebar) */
     menubar = gtk_ui_manager_get_widget(UIManager, "/MenuBar");
     Init_Menu_Bar();
     gtk_widget_show_all(menubar);
 
-    /* Toolbar is placed in a handlebar */
-    toolbar = gtk_handle_box_new();
-    toolbarwidget = gtk_ui_manager_get_widget(UIManager, "/ToolBar");
-    gtk_container_add(GTK_CONTAINER(toolbar), toolbarwidget);
-    gtk_toolbar_set_style(GTK_TOOLBAR(toolbarwidget), GTK_TOOLBAR_ICONS);
+    toolbar = gtk_ui_manager_get_widget (UIManager, "/ToolBar");
+    gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
     gtk_widget_show_all(toolbar);
 
     *pptoolbar = toolbar;
@@ -350,7 +453,8 @@ Init_Menu_Bar (void)
             G_CALLBACK(Check_Menu_Item_Toggled_Browse_Subdir),NULL);
     }
 
-    CheckMenuItemBrowseHiddenDirMainMenu = gtk_ui_manager_get_widget(UIManager, "/MenuBar/BrowserMenu/BrowseHiddenDir");
+    CheckMenuItemBrowseHiddenDirMainMenu = gtk_ui_manager_get_widget (UIManager,
+                                                                      "/MenuBar/ViewMenu/BrowseHiddenDir");
     if (CheckMenuItemBrowseHiddenDirMainMenu)
     {
         // Link to update BROWSE_HIDDEN_DIR when changed
@@ -416,8 +520,12 @@ GtkWidget *Create_Status_Bar (void)
     gtk_widget_set_size_request(StatusBar, 200, -1);
     /* Create serie */
     StatusBarContext = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar),"Messages");
+    timer_cid = gtk_statusbar_get_context_id (GTK_STATUSBAR (StatusBar),
+                                              "timer");
+    tooltip_cid = gtk_statusbar_get_context_id (GTK_STATUSBAR (StatusBar),
+                                                "tooltip");
 
-    Statusbar_Message(_("Ready to start…"),TRUE);
+    Statusbar_Message (_("Ready to start"), TRUE);
 
     gtk_widget_show(StatusBar);
     return StatusBar;
@@ -426,15 +534,18 @@ GtkWidget *Create_Status_Bar (void)
 static gboolean
 Statusbar_Stop_Timer (void)
 {
-    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar),StatusBarContext);
-    return FALSE;    /* Stop the timer */
+    gtk_statusbar_pop (GTK_STATUSBAR (StatusBar), timer_cid);
+    return G_SOURCE_REMOVE;
 }
 
 static void
 Statusbar_Start_Timer (void)
 {
     Statusbar_Remove_Timer ();
-    StatusbarTimerId = g_timeout_add(4000,(GSourceFunc)Statusbar_Stop_Timer,NULL);
+    StatusbarTimerId = g_timeout_add_seconds (4,
+                                              (GSourceFunc)Statusbar_Stop_Timer,
+                                              NULL);
+    g_source_set_name_by_id (StatusbarTimerId, "Statusbar stop timer");
 }
 
 static void
@@ -442,6 +553,7 @@ Statusbar_Remove_Timer (void)
 {
     if (StatusbarTimerId)
     {
+        Statusbar_Stop_Timer ();
         g_source_remove(StatusbarTimerId);
         StatusbarTimerId = 0;
     }
@@ -452,39 +564,59 @@ Statusbar_Remove_Timer (void)
  *  - with_timer: if TRUE, the message will be displayed during 4s
  *                if FALSE, the message will be displayed up to the next posted message
  */
-void Statusbar_Message (gchar *message, gint with_timer)
+void
+Statusbar_Message (const gchar *message, gboolean with_timer)
 {
     gchar *msg_temp;
 
     g_return_if_fail (StatusBar != NULL);
 
-    /* Validate UTF8 */
-    /*if (!g_utf8_validate(message, -1, NULL))
-        msg_temp = convert_to_utf8(message);
-    else
-        msg_temp = g_strdup(message);*/
     msg_temp = Try_To_Validate_Utf8_String(message);
     
-    /* Remove a running timer */
-    Statusbar_Remove_Timer();
-
-    /* Pop last message */
-    gtk_statusbar_pop(GTK_STATUSBAR(StatusBar),StatusBarContext);
-
     /* Push the given message */
-    gtk_statusbar_push(GTK_STATUSBAR(StatusBar),StatusBarContext,msg_temp);
+    if (with_timer)
+    {
+        Statusbar_Start_Timer ();
+        gtk_statusbar_push (GTK_STATUSBAR (StatusBar), timer_cid, msg_temp);
+    }
+    else
+    {
+        gtk_statusbar_pop (GTK_STATUSBAR (StatusBar), StatusBarContext);
+        gtk_statusbar_push (GTK_STATUSBAR (StatusBar), StatusBarContext,
+                            msg_temp);
+    }
 
     g_free(msg_temp);
-
-    while (gtk_events_pending())
-        gtk_main_iteration();
-
-    if (with_timer)
-        Statusbar_Start_Timer();
 }
 
+/*
+ * et_statusbar_push_tooltip:
+ * @message: a tooltip to display in the status bar
+ *
+ * Display a tooltip in the status bar of the main window. Call
+ * et_statusbar_pop_tooltip() to stop displaying the tooltip message.
+ */
+static void
+et_statusbar_push_tooltip (const gchar *message)
+{
+    g_return_if_fail (StatusBar != NULL && message != NULL);
 
+    gtk_statusbar_push (GTK_STATUSBAR (StatusBar), tooltip_cid, message);
+}
 
+/*
+ * et_statusbar_pop_tooltip:
+ *
+ * Pop a tooltip message from the status bar. et_statusbar_push_tooltip() must
+ * have been called first.
+ */
+static void
+et_statusbar_pop_tooltip (void)
+{
+    g_return_if_fail (StatusBar != NULL);
+
+    gtk_statusbar_pop (GTK_STATUSBAR (StatusBar), tooltip_cid);
+}
 
 
 
@@ -499,3 +631,97 @@ GtkWidget *Create_Progress_Bar (void)
     gtk_widget_show(ProgressBar);
     return ProgressBar;
 }
+
+/*
+ * et_ui_manager_on_connect_proxy:
+ * @manager: the UI manager which generated the signal
+ * @action: the action which was connected to @proxy
+ * @proxy: the widget which was connected to @action
+ * @user_data: user data set when the signal was connected
+ *
+ * Connect handlers for selection and deselection of menu items, in order to
+ * set tooltips for the menu items as status bar messages.
+ */
+static void
+et_ui_manager_on_connect_proxy (GtkUIManager *manager, GtkAction *action,
+                                GtkWidget *proxy, gpointer user_data)
+{
+    if (GTK_IS_MENU_ITEM (proxy))
+    {
+        guint id;
+
+        id = g_signal_connect (proxy, "select",
+                               G_CALLBACK (on_menu_item_select), action);
+        g_object_set_data (G_OBJECT (proxy), "select-id",
+                           GUINT_TO_POINTER (id));
+        id = g_signal_connect (proxy, "deselect",
+                               G_CALLBACK (on_menu_item_deselect), NULL);
+        g_object_set_data (G_OBJECT (proxy), "deselect-id",
+                           GUINT_TO_POINTER (id));
+    }
+}
+
+/*
+ * et_ui_manager_on_disconnect_proxy:
+ * @manager: the UI manager which generated the signal
+ * @action: the action which was connected to @proxy
+ * @proxy: the widget which was connected to @action
+ * @user_data: user data set when the signal was connected
+ *
+ * Disconnect handlers for selecting and deselecting menu items.
+ */
+static void
+et_ui_manager_on_disconnect_proxy (GtkUIManager *manager, GtkAction *action,
+                                   GtkWidget *proxy, gpointer user_data)
+{
+    if (GTK_IS_MENU_ITEM (proxy))
+    {
+        guint id;
+
+        id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (proxy),
+                                                  "select-id"));
+        g_signal_handler_disconnect (proxy, id);
+        id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (proxy),
+                                                  "deselect-id"));
+        g_signal_handler_disconnect (proxy, id);
+    }
+}
+
+/*
+ * on_menu_item_select:
+ * @item: the menu item which was selected
+ * @user_data: the #GtkAction corresponding to @item
+ *
+ * Set the current status bar message to the tooltip of the menu @item which
+ * was selected.
+ */
+static void
+on_menu_item_select (GtkMenuItem *item, gpointer user_data)
+{
+    GtkAction *action;
+    const gchar *message;
+
+    g_return_if_fail (user_data != NULL);
+
+    action = GTK_ACTION (user_data);
+    message = gtk_action_get_tooltip (action);
+
+    if (message)
+    {
+        et_statusbar_push_tooltip (message);
+    }
+}
+
+/*
+ * on_menu_item_deselect:
+ * @item: the menu item which was deselected
+ * @user_data: user data set when the signal was connected
+ *
+ * Clear the current tooltip status bar message when the menu item is
+ * deselected.
+ */
+static void
+on_menu_item_deselect (GtkMenuItem *item, gpointer user_data)
+{
+    et_statusbar_pop_tooltip ();
+}
