@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -40,18 +40,7 @@
 gboolean Main_Stop_Button_Pressed;
 
 GtkWidget *MainWindow;
-GtkWidget *MenuArea;
-GtkWidget *ToolArea;
-GtkWidget *BrowseArea;
-GtkWidget *FileArea;
-GtkWidget *TagArea;
-GtkWidget *StatusArea;
-GtkWidget *ProgressArea;
 GtkWidget *LogArea;
-GtkWidget *ReadOnlyStatusIconBox;
-GtkWidget *BrokenStatusIconBox;
-GtkWidget *MainWindowHPaned;
-GtkWidget *MainWindowVPaned;
 
 /* File Area */
 GtkWidget *FileFrame;
@@ -93,50 +82,12 @@ GtkWidget    *CopyrightEntry;
 GtkWidget    *URLEntry;
 GtkWidget    *EncodedByEntry;
 GtkWidget    *PictureEntryView;
-GtkListStore *PictureEntryModel;
-// Labels
-GtkWidget    *TitleLabel;
-GtkWidget    *ArtistLabel;
-GtkWidget    *AlbumArtistLabel;
-GtkWidget    *AlbumLabel;
-GtkWidget    *DiscNumberLabel;
-GtkWidget    *YearLabel;
-GtkWidget    *TrackLabel;
-GtkWidget    *GenreLabel;
-GtkWidget    *CommentLabel;
-GtkWidget    *ComposerLabel;
-GtkWidget    *OrigArtistLabel;
-GtkWidget    *CopyrightLabel;
-GtkWidget    *URLLabel;
-GtkWidget    *EncodedByLabel;
-GtkWidget    *PictureLabel;
-// Mini buttons
-GtkWidget    *TitleMButton;
-GtkWidget    *ArtistMButton;
-GtkWidget    *AlbumArtistMButton;
-GtkWidget    *AlbumMButton;
-GtkWidget    *DiscNumberMButton;
-GtkWidget    *YearMButton;
-GtkWidget    *TrackMButton;
-GtkWidget    *TrackMButtonSequence;
-GtkWidget    *TrackMButtonNbrFiles;
-GtkWidget    *GenreMButton;
-GtkWidget    *CommentMButton;
-GtkWidget    *ComposerMButton;
-GtkWidget    *OrigArtistMButton;
-GtkWidget    *CopyrightMButton;
-GtkWidget    *URLMButton;
-GtkWidget    *EncodedByMButton;
-GtkWidget    *PictureMButton;
 
-// Other for picture
-GtkWidget *PictureClearButton;
-GtkWidget *PictureAddButton;
-GtkWidget *PictureSaveButton;
-GtkWidget *PicturePropertiesButton;
-GtkWidget *PictureScrollWindow;
-
-GdkCursor *MouseCursor;
+/* Other for picture. */
+GtkToolItem *remove_image_toolitem;
+GtkToolItem *add_image_toolitem;
+GtkToolItem *save_image_toolitem;
+GtkToolItem *image_properties_toolitem;
 
 gchar *INIT_DIRECTORY;
 
@@ -151,8 +102,8 @@ gboolean ReadingDirectory;
 /**************
  * Prototypes *
  **************/
-void Action_Select_All_Files            (void);
-void Action_Unselect_All_Files          (void);
+void et_on_action_select_all (void);
+void et_on_action_unselect_all (void);
 void Action_Invert_Files_Selection      (void);
 void Action_Select_Prev_File            (void);
 void Action_Select_Next_File            (void);
@@ -176,6 +127,9 @@ gint Save_All_Files_With_Answer         (gboolean force_saving_files);
 
 void Action_Main_Stop_Button_Pressed    (void);
 void Action_Select_Browser_Style        (void);
+void et_on_action_select_scan_mode (GtkRadioAction *action,
+                                    GtkRadioAction *current,
+                                    gpointer user_data);
 
 void Tag_Area_Display_Controls (ET_File *ETFile);
 
