@@ -16,13 +16,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
 
 #include <gtk/gtk.h>
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <errno.h>
@@ -36,16 +36,13 @@
 #include "picture.h"
 #include "easytag.h"
 #include "browser.h"
-#include "genres.h"
 #include "setting.h"
 #include "misc.h"
 #include "log.h"
 #include "et_core.h"
 #include "charset.h"
 
-#ifdef G_OS_WIN32
 #include "win32/win32dep.h"
-#endif /* G_OS_WIN32 */
 
 
 #ifdef ENABLE_MP3
@@ -93,7 +90,7 @@ static int etag_write_tags (const gchar *filename, struct id3_tag const *v1tag,
  * Returns TRUE on success, else FALSE.
  * If a tag entry exists (ex: title), we allocate memory, else value stays to NULL
  */
-gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
+gboolean Id3tag_Read_File_Tag (const gchar *filename, File_Tag *FileTag)
 {
     int tmpfile;
     struct id3_file *file;
@@ -107,13 +104,13 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     long tagsize;
 
 
-    if (!filename || !FileTag)
-        return FALSE;
+    g_return_val_if_fail (filename != NULL && FileTag != NULL, FALSE);
 
     if ( (tmpfile=open(filename,O_RDONLY)) < 0 )
     {
         gchar *filename_utf8 = filename_to_display(filename);
-        g_print(_("Error while opening file: '%s' (%s).\n\a"),filename_utf8,g_strerror(errno));
+        Log_Print (LOG_ERROR, _("Error while opening file: '%s' (%s)."),
+                   filename_utf8, g_strerror (errno));
         g_free(filename_utf8);
         return FALSE;
     }
@@ -235,8 +232,25 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     /************************
      * Part of a set (TPOS) *
      ************************/
-    if ( (frame = id3_tag_findframe(tag,"TPOS", 0)) )
-        update |= libid3tag_Get_Frame_Str(frame, ~0, &FileTag->disc_number);
+    if ((frame = id3_tag_findframe (tag, "TPOS", 0)))
+    {
+        update |= libid3tag_Get_Frame_Str (frame, ~0, &string1);
+
+        if (string1)
+        {
+            string2 = g_utf8_strchr (string1, -1, '/');
+
+            if (string2)
+            {
+                FileTag->disc_total = et_disc_number_to_string (atoi (string2
+                                                                      + 1));
+                *string2 = '\0';
+            }
+
+            FileTag->disc_number = et_disc_number_to_string (atoi (string1));
+            g_free (string1);
+        }
+    }
 
     /********************
      * Year (TYER/TDRC) *
@@ -246,7 +260,7 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
         update |= libid3tag_Get_Frame_Str(frame, ~0, &string1);
         if ( string1 )
         {
-            Strip_String(string1);
+            g_strstrip (string1);
             FileTag->year = string1;
         }
     }
@@ -261,23 +275,13 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
         {
             string2 = g_utf8_strchr(string1,-1,'/');
 
-            if (NUMBER_TRACK_FORMATED)
+            if (string2)
             {
-                if (string2)
-                {
-                    FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string2+1)); // Just to have numbers like this : '01', '05', '12', ...
-                    *string2 = '\0'; // To cut string1
-                }
-                FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string1)); // Just to have numbers like this : '01', '05', '12', ...
-            }else
-            {
-                if (string2)
-                {
-                    FileTag->track_total = g_strdup(string2+1);
-                    *string2 = '\0'; // To cut string1
-                }
-                FileTag->track = g_strdup(string1);
+                FileTag->track_total = et_track_number_to_string (atoi (string2 + 1));
+                *string2 = '\0'; // To cut string1
             }
+            FileTag->track = et_track_number_to_string (atoi (string1));
+
             g_free(string1);
 
         }
@@ -780,9 +784,7 @@ gboolean Id3tag_Write_File_v24Tag (ET_File *ETFile)
     gboolean strip_tags = TRUE;
     guchar genre_value = ID3_INVALID_GENRE;
 
-
-    if (!ETFile && !ETFile->FileTag)
-        return FALSE;
+    g_return_val_if_fail (ETFile != NULL && ETFile->FileTag != NULL, FALSE);
 
     FileTag       = (File_Tag *)ETFile->FileTag->data;
     filename      = ((File_Name *)ETFile->FileNameCur->data)->value;
@@ -890,7 +892,19 @@ gboolean Id3tag_Write_File_v24Tag (ET_File *ETFile)
     /***************
      * Part of set *
      ***************/
-    etag_set_tags(FileTag->disc_number, "TPOS", ID3_FIELD_TYPE_STRINGLIST, NULL, v2tag, &strip_tags);
+    if (FileTag->disc_number && FileTag->disc_total && *FileTag->disc_total)
+    {
+        string1 = g_strconcat (FileTag->disc_number, "/", FileTag->disc_total,
+                               NULL);
+    }
+    else
+    {
+        string1 = g_strdup (FileTag->disc_number);
+    }
+
+    etag_set_tags (string1, "TPOS", ID3_FIELD_TYPE_STRINGLIST, NULL, v2tag,
+                   &strip_tags);
+    g_free (string1);
 
     /********
      * Year *
@@ -1101,8 +1115,7 @@ Id3tag_find_and_create_frame (struct id3_tag *tag, const gchar *name)
 {
     struct id3_frame *frame;
 
-    if (!tag || !name || !*name)
-        return NULL;
+    g_return_val_if_fail (tag != NULL && name != NULL && *name != 0, NULL);
 
     frame = id3_tag_findframe(tag, name, 0);
     if (!frame)
@@ -1323,10 +1336,11 @@ etag_write_tags (const gchar *filename,
     char tmp[ID3_TAG_QUERYSIZE];
     int fd;
     int curpos;
-    long filev2size, ctxsize;
+    long filev2size;
+    gsize ctxsize;
     char *ctx = NULL;
     int err = 0;
-    long size_read = 0;
+    gssize size_read = 0;
 
     v1buf = v2buf = NULL;
     if ( !strip_tags )
@@ -1474,10 +1488,26 @@ etag_write_tags (const gchar *filename,
         {
             gchar *filename_utf8 = filename_to_display(filename);
             gchar *basename_utf8 = g_path_get_basename(filename_utf8);
+            gchar *bytes_missing = g_strdup_printf ("%" G_GSIZE_FORMAT,
+                                                    ctxsize - size_read);
 
-            Log_Print(LOG_ERROR,_("Cannot write tag of file '%s' (%d bytes were read but %d bytes were expected)"),basename_utf8,size_read,ctxsize);
+            /* Construct the string in this awkward way as xgettext does not
+             * support macros in extracted strings.
+             * https://bugzilla.gnome.org/show_bug.cgi?id=705952
+             */
+            Log_Print (LOG_ERROR,
+                       /* Translators: The first string is a filename, the
+                        * second string is the number of bytes that were
+                        * missing (not read for some reason) while reading from
+                        * the file.
+                        */
+                       ngettext ("Cannot write tag of file ‘%s’ (a byte was missing)",
+                                 "Cannot write tag of file ‘%s’ (%s bytes were missing)",
+                                 ctxsize - size_read),
+                       basename_utf8, bytes_missing);
             g_free(filename_utf8);
             g_free(basename_utf8);
+            g_free (bytes_missing);
             goto out;
         }
         
@@ -1487,8 +1517,17 @@ etag_write_tags (const gchar *filename,
             
         // Write the ID3v2 tag
         if (v2buf)
-            write(fd, v2buf, v2size);
-        
+        {
+            if (write (fd, v2buf, v2size) != v2size)
+            {
+                gchar *filename_utf8 = filename_to_display (filename);
+                gchar *basename_utf8 = g_path_get_basename (filename_utf8);
+                Log_Print (LOG_ERROR, _("Cannot save tag of file '%s'"),
+                           basename_utf8);
+                g_free (basename_utf8);
+                goto out;
+            }
+        }
         // Write audio data
         if (write(fd, ctx, ctxsize) != ctxsize)
         {
