@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
 
 #include <gtk/gtk.h>
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -42,10 +42,9 @@
 #include "misc.h"
 #include "cddb.h"
 #include "browser.h"
+#include "et_core.h"
 
-#ifdef G_OS_WIN32
 #include "win32/win32dep.h"
-#endif /* G_OS_WIN32 */
 
 
 /***************
@@ -101,13 +100,16 @@ static const gchar CDDB_LOCAL_PATH_HISTORY_FILE[] = "cddb_local_path.history";
 
 static void Save_Config_To_File (void);
 static gboolean Create_Easytag_Directory (void);
+static void set_sorting_indicator_for_column_id (gint column_id,
+                                                 ET_Sorting_Type temp_sort,
+                                                 GtkTreeViewColumn *column);
 
 
 
 /********************
  * Config Variables *
  ********************/
-tConfigVariable Config_Variables[] =
+static const tConfigVariable Config_Variables[] =
 {
     {"load_on_startup",                     CV_TYPE_BOOL,    &LOAD_ON_STARTUP                   },
     {"default_path_to_mp3",                 CV_TYPE_STRING,  &DEFAULT_PATH_TO_MP3               },
@@ -133,7 +135,8 @@ tConfigVariable Config_Variables[] =
     {"date_auto_completion",                 CV_TYPE_BOOL,    &DATE_AUTO_COMPLETION                     },
     {"number_track_formated",                CV_TYPE_BOOL,    &NUMBER_TRACK_FORMATED                    },
     {"number_track_formated_spin_button",    CV_TYPE_INT,     &NUMBER_TRACK_FORMATED_SPIN_BUTTON        },
-    {"ogg_tag_write_xmms_comment",           CV_TYPE_BOOL,    &OGG_TAG_WRITE_XMMS_COMMENT               },
+    {"pad_disc_number", CV_TYPE_BOOL, &PAD_DISC_NUMBER },
+    {"pad_disc_number_digits", CV_TYPE_INT, &PAD_DISC_NUMBER_DIGITS },
     {"set_focus_to_same_tag_field",          CV_TYPE_BOOL,    &SET_FOCUS_TO_SAME_TAG_FIELD              },
     {"set_focus_to_first_tag_field",         CV_TYPE_BOOL,    &SET_FOCUS_TO_FIRST_TAG_FIELD             },
     {"sorting_file_mode",                    CV_TYPE_INT,     &SORTING_FILE_MODE                        },
@@ -151,7 +154,6 @@ tConfigVariable Config_Variables[] =
     {"filename_character_set_approximate",             CV_TYPE_BOOL,    &FILENAME_CHARACTER_SET_APPROXIMATE       },
     {"filename_character_set_discard",                 CV_TYPE_BOOL,    &FILENAME_CHARACTER_SET_DISCARD           },
 
-    {"write_id3_tags_in_flac_file",                    CV_TYPE_BOOL,  &WRITE_ID3_TAGS_IN_FLAC_FILE                     },
     {"strip_tag_when_empty_fields",                    CV_TYPE_BOOL,  &STRIP_TAG_WHEN_EMPTY_FIELDS                     },
     {"convert_old_id3v2_tag_version",                  CV_TYPE_BOOL,  &CONVERT_OLD_ID3V2_TAG_VERSION                   },
     {"use_non_standard_id3_reading_character_set",     CV_TYPE_BOOL,  &USE_NON_STANDARD_ID3_READING_CHARACTER_SET},
@@ -180,11 +182,6 @@ tConfigVariable Config_Variables[] =
     {"vorbis_split_field_composer",                    CV_TYPE_BOOL,  &VORBIS_SPLIT_FIELD_COMPOSER},
     {"vorbis_split_field_orig_artist",                 CV_TYPE_BOOL,  &VORBIS_SPLIT_FIELD_ORIG_ARTIST},
 
-    {"message_box_position_none",               CV_TYPE_BOOL,  &MESSAGE_BOX_POSITION_NONE                },
-    {"message_box_position_center",             CV_TYPE_BOOL,  &MESSAGE_BOX_POSITION_CENTER              },
-    {"message_box_position_mouse",              CV_TYPE_BOOL,  &MESSAGE_BOX_POSITION_MOUSE               },
-    {"message_box_position_center_on_parent",   CV_TYPE_BOOL,  &MESSAGE_BOX_POSITION_CENTER_ON_PARENT    },
-
     {"audio_file_player",                       CV_TYPE_STRING,&AUDIO_FILE_PLAYER                        },
 
     {"scanner_type",                             CV_TYPE_INT, &SCANNER_TYPE                              },
@@ -201,7 +198,6 @@ tConfigVariable Config_Variables[] =
     {"default_comment",                         CV_TYPE_STRING,  &DEFAULT_COMMENT                        },
     {"crc32_comment",                           CV_TYPE_BOOL,    &SET_CRC32_COMMENT                      },
     {"open_scanner_window_on_startup",          CV_TYPE_BOOL,    &OPEN_SCANNER_WINDOW_ON_STARTUP         },
-    {"scanner_window_on_top",                   CV_TYPE_BOOL,    &SCANNER_WINDOW_ON_TOP                  },
     {"set_scanner_window_position",             CV_TYPE_BOOL,    &SET_SCANNER_WINDOW_POSITION            },
     {"scanner_window_x",                        CV_TYPE_INT,     &SCANNER_WINDOW_X                       },
     {"scanner_window_y",                        CV_TYPE_INT,     &SCANNER_WINDOW_Y                       },
@@ -212,6 +208,7 @@ tConfigVariable Config_Variables[] =
     {"confirm_write_playlist",                  CV_TYPE_BOOL,    &CONFIRM_WRITE_PLAYLIST                 },
     {"confirm_delete_file",                     CV_TYPE_BOOL,    &CONFIRM_DELETE_FILE                    },
     {"confirm_when_unsaved_files",              CV_TYPE_BOOL,    &CONFIRM_WHEN_UNSAVED_FILES             },
+
     {"process_filename_field",                  CV_TYPE_BOOL,    &PROCESS_FILENAME_FIELD                 },
     {"process_title_field",                     CV_TYPE_BOOL,    &PROCESS_TITLE_FIELD                    },
     {"process_artist_field",                    CV_TYPE_BOOL,    &PROCESS_ARTIST_FIELD                   },
@@ -253,8 +250,6 @@ tConfigVariable Config_Variables[] =
     {"playlist_content_mask_value",             CV_TYPE_STRING,  &PLAYLIST_CONTENT_MASK_VALUE            },
     {"playlist_window_x",                       CV_TYPE_INT,     &PLAYLIST_WINDOW_X                      },
     {"playlist_window_y",                       CV_TYPE_INT,     &PLAYLIST_WINDOW_Y                      },
-    {"playlist_window_width",                   CV_TYPE_INT,     &PLAYLIST_WINDOW_WIDTH                  },
-    {"playlist_window_height",                  CV_TYPE_INT,     &PLAYLIST_WINDOW_HEIGHT                 },
 
     {"load_file_run_scanner",                   CV_TYPE_BOOL,    &LOAD_FILE_RUN_SCANNER                  },
     {"load_file_window_x",                      CV_TYPE_INT,     &LOAD_FILE_WINDOW_X                     },
@@ -333,10 +328,7 @@ tConfigVariable Config_Variables[] =
     {"rename_directory_default_mask",           CV_TYPE_STRING,  &RENAME_DIRECTORY_DEFAULT_MASK          },
     {"rename_directory_with_mask",              CV_TYPE_BOOL,    &RENAME_DIRECTORY_WITH_MASK             },
 
-    {"options_notebook_page",                   CV_TYPE_INT,     &OPTIONS_NOTEBOOK_PAGE                  },
-    {"options_window_height",                   CV_TYPE_INT,     &OPTIONS_WINDOW_HEIGHT                  },
-    {"options_window_width",                    CV_TYPE_INT,     &OPTIONS_WINDOW_WIDTH                   }
-
+    { "options_notebook_page", CV_TYPE_INT, &OPTIONS_NOTEBOOK_PAGE }
 };
 
 
@@ -388,7 +380,8 @@ void Init_Config_Variables (void)
     DATE_AUTO_COMPLETION                    = 1;
     NUMBER_TRACK_FORMATED                   = 1;
     NUMBER_TRACK_FORMATED_SPIN_BUTTON       = 2;
-    OGG_TAG_WRITE_XMMS_COMMENT              = 1;
+    PAD_DISC_NUMBER = 1;
+    PAD_DISC_NUMBER_DIGITS = 1;
     SET_FOCUS_TO_SAME_TAG_FIELD             = 1;
     SET_FOCUS_TO_FIRST_TAG_FIELD            = 0;
     SORTING_FILE_MODE                       = SORTING_BY_ASCENDING_FILENAME;
@@ -400,15 +393,10 @@ void Init_Config_Variables (void)
     LOG_MAX_LINES                           = 50;
     SHOW_LOG_VIEW                           = 1;
 
-    MESSAGE_BOX_POSITION_NONE               = 0;
-    MESSAGE_BOX_POSITION_CENTER             = 0;
-    MESSAGE_BOX_POSITION_MOUSE              = 0;
-    MESSAGE_BOX_POSITION_CENTER_ON_PARENT   = 1;
-
 #ifdef G_OS_WIN32
     AUDIO_FILE_PLAYER                       = ET_Win32_Get_Audio_File_Player();
 #else /* !G_OS_WIN32 */
-    AUDIO_FILE_PLAYER                       = g_strdup("xmms -p");
+    AUDIO_FILE_PLAYER                       = g_strdup("xdg-open");
 #endif /* !G_OS_WIN32 */
 
     /*
@@ -418,8 +406,8 @@ void Init_Config_Variables (void)
     FILENAME_EXTENSION_LOWER_CASE               = 1;
     FILENAME_EXTENSION_UPPER_CASE               = 0;
     FILENAME_EXTENSION_NO_CHANGE                = 0;
-    PRESERVE_MODIFICATION_TIME                  = 1;
-    UPDATE_PARENT_DIRECTORY_MODIFICATION_TIME   = 0;
+    PRESERVE_MODIFICATION_TIME                  = 0;
+    UPDATE_PARENT_DIRECTORY_MODIFICATION_TIME   = 1;
 
     FILENAME_CHARACTER_SET_OTHER                = 1;
     FILENAME_CHARACTER_SET_APPROXIMATE          = 0;
@@ -428,7 +416,6 @@ void Init_Config_Variables (void)
     /*
      * Tag Settings
      */
-    WRITE_ID3_TAGS_IN_FLAC_FILE                     = 0;
     STRIP_TAG_WHEN_EMPTY_FIELDS                     = 1;
     CONVERT_OLD_ID3V2_TAG_VERSION                   = 1;
     USE_NON_STANDARD_ID3_READING_CHARACTER_SET      = 0;
@@ -457,13 +444,13 @@ void Init_Config_Variables (void)
     FILE_WRITING_ID3V1_ICONV_OPTIONS_TRANSLIT       = 1;
     FILE_WRITING_ID3V1_ICONV_OPTIONS_IGNORE         = 0;
 
-    VORBIS_SPLIT_FIELD_TITLE                          = 1;
-    VORBIS_SPLIT_FIELD_ARTIST                         = 1;
-    VORBIS_SPLIT_FIELD_ALBUM                          = 1;
-    VORBIS_SPLIT_FIELD_GENRE                          = 1;
-    VORBIS_SPLIT_FIELD_COMMENT                        = 1;
-    VORBIS_SPLIT_FIELD_COMPOSER                       = 1;
-    VORBIS_SPLIT_FIELD_ORIG_ARTIST                    = 1;
+    VORBIS_SPLIT_FIELD_TITLE                          = 0;
+    VORBIS_SPLIT_FIELD_ARTIST                         = 0;
+    VORBIS_SPLIT_FIELD_ALBUM                          = 0;
+    VORBIS_SPLIT_FIELD_GENRE                          = 0;
+    VORBIS_SPLIT_FIELD_COMMENT                        = 0;
+    VORBIS_SPLIT_FIELD_COMPOSER                       = 0;
+    VORBIS_SPLIT_FIELD_ORIG_ARTIST                    = 0;
     /*
      * Scanner
      */
@@ -480,7 +467,6 @@ void Init_Config_Variables (void)
     DEFAULT_COMMENT                           = g_strdup("Tagged with EasyTAG");
     SET_CRC32_COMMENT                         = 0;
     OPEN_SCANNER_WINDOW_ON_STARTUP            = 0;
-    SCANNER_WINDOW_ON_TOP                     = 1;
     SET_SCANNER_WINDOW_POSITION               = 1; // Set it to '0' if problem with some Windows Manager
     SCANNER_WINDOW_X                          = -1;
     SCANNER_WINDOW_Y                          = -1;
@@ -543,8 +529,6 @@ void Init_Config_Variables (void)
 
     PLAYLIST_WINDOW_X               = -1;
     PLAYLIST_WINDOW_Y               = -1;
-    PLAYLIST_WINDOW_WIDTH           = -1;
-    PLAYLIST_WINDOW_HEIGHT          = -1;
 
     /*
      * Load File window
@@ -640,9 +624,6 @@ void Init_Config_Variables (void)
      * Other parameters
      */
     OPTIONS_NOTEBOOK_PAGE = 0;
-    OPTIONS_WINDOW_HEIGHT = 300;
-    OPTIONS_WINDOW_WIDTH  = 400;
-
 }
 
 
@@ -657,6 +638,9 @@ Apply_Changes_Of_Preferences_Window (void)
 {
     gchar *temp;
     int active;
+    ET_Sorting_Type temp_sort;
+    gint column_id;
+    GtkTreeViewColumn * column;
 
     if (OptionsWindow)
     {
@@ -687,19 +671,28 @@ Apply_Changes_Of_Preferences_Window (void)
         DATE_AUTO_COMPLETION                   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(DateAutoCompletion));
         NUMBER_TRACK_FORMATED                  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(NumberTrackFormated));
         NUMBER_TRACK_FORMATED_SPIN_BUTTON      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(NumberTrackFormatedSpinButton));
-        OGG_TAG_WRITE_XMMS_COMMENT             = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(OggTagWriteXmmsComment));
+        PAD_DISC_NUMBER = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pad_disc_number));
+        PAD_DISC_NUMBER_DIGITS = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pad_disc_number_spinbutton));
         SORTING_FILE_CASE_SENSITIVE            = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SortingFileCaseSensitive));
         SET_FOCUS_TO_SAME_TAG_FIELD            = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SetFocusToSameTagField));
         SET_FOCUS_TO_FIRST_TAG_FIELD           = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SetFocusToFirstTagField));
         LOG_MAX_LINES                          = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(LogMaxLinesSpinButton));
         SHOW_LOG_VIEW                          = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ShowLogView));
 
-        SORTING_FILE_MODE = gtk_combo_box_get_active(GTK_COMBO_BOX(SortingFileCombo));
-
-        MESSAGE_BOX_POSITION_NONE               = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(MessageBoxPositionNone));
-        MESSAGE_BOX_POSITION_CENTER             = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(MessageBoxPositionCenter));
-        MESSAGE_BOX_POSITION_MOUSE              = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(MessageBoxPositionMouse));
-        MESSAGE_BOX_POSITION_CENTER_ON_PARENT   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(MessageBoxPositionCenterOnParent));
+        temp_sort = gtk_combo_box_get_active (GTK_COMBO_BOX (SortingFileCombo));
+        column_id = temp_sort / 2;
+        if (SORTING_FILE_MODE < SORTING_BY_ASCENDING_CREATION_DATE)
+        {
+            gtk_tree_view_column_set_sort_indicator (get_column_for_column_id (SORTING_FILE_MODE / 2),
+                                                     FALSE);
+        }
+        if (temp_sort < SORTING_BY_ASCENDING_CREATION_DATE)
+        {
+            column = get_column_for_column_id (column_id);
+            set_sorting_indicator_for_column_id (column_id, temp_sort, column);
+        }
+        SORTING_FILE_MODE = temp_sort;
+        Browser_List_Refresh_Sort ();
 
         if (AUDIO_FILE_PLAYER) g_free(AUDIO_FILE_PLAYER);
         AUDIO_FILE_PLAYER                       = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(FilePlayerCombo)))));
@@ -717,7 +710,6 @@ Apply_Changes_Of_Preferences_Window (void)
         FILENAME_CHARACTER_SET_DISCARD            = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(FilenameCharacterSetDiscard));
 
         /* Tag Settings */
-        WRITE_ID3_TAGS_IN_FLAC_FILE                = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(WriteId3TagsInFlacFiles));
         STRIP_TAG_WHEN_EMPTY_FIELDS                = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(StripTagWhenEmptyFields));
         CONVERT_OLD_ID3V2_TAG_VERSION              = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ConvertOldId3v2TagVersion));
         USE_NON_STANDARD_ID3_READING_CHARACTER_SET = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(UseNonStandardId3ReadingCharacterSet));
@@ -784,7 +776,6 @@ Apply_Changes_Of_Preferences_Window (void)
         SET_CRC32_COMMENT   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Crc32Comment));
 
         OPEN_SCANNER_WINDOW_ON_STARTUP = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(OpenScannerWindowOnStartup));
-        SCANNER_WINDOW_ON_TOP          = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ScannerWindowOnTop));
 
         /* CDDB */
         if (CDDB_SERVER_NAME_AUTOMATIC_SEARCH) g_free(CDDB_SERVER_NAME_AUTOMATIC_SEARCH);
@@ -859,12 +850,9 @@ Apply_Changes_Of_Preferences_Window (void)
 
     if (ScannerWindow)
     {
-        if (SCANNER_WINDOW_ON_TOP)
-            gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),GTK_WINDOW(MainWindow));
-        else
-            gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),NULL);
+        gtk_window_set_transient_for (GTK_WINDOW (ScannerWindow),
+                                      GTK_WINDOW (MainWindow));
     }
-
 }
 
 /*
@@ -950,8 +938,16 @@ Save_Config_To_File (void)
                 {
                     data = g_strdup_printf("%s=%i\n",Config_Variables[i].name,
                                                      *(int *)Config_Variables[i].pointer);
-                    fwrite(data,strlen(data),1,file);
-                    //g_print("# (type:%d) %s",Config_Variables[i].type,data);
+                    if (fwrite (data, strlen (data), 1, file) != 1)
+                    {
+                        Log_Print (LOG_ERROR,
+                                   _("Error while writing configuration file: %s"),
+                                   file_path);
+                        fclose (file);
+                        g_free (file_path);
+                        g_free (data);
+                        return;
+                    }
                     g_free(data);
                     break;
                 }
@@ -959,8 +955,16 @@ Save_Config_To_File (void)
                 {
                     data = g_strdup_printf("%s=%i\n",Config_Variables[i].name,
                                                      ( *(int *)Config_Variables[i].pointer ? 1 : 0 ));
-                    fwrite(data,strlen(data),1,file);
-                    //g_print("# (type:%d) %s",Config_Variables[i].type,data);
+                    if (fwrite (data, strlen (data), 1, file) != 1)
+                    {
+                        Log_Print (LOG_ERROR,
+                                   _("Error while writing configuration file: %s"),
+                                   file_path);
+                        fclose (file);
+                        g_free (file_path);
+                        g_free (data);
+                        return;
+                    }
                     g_free(data);
                     break;
                 }
@@ -971,8 +975,16 @@ Save_Config_To_File (void)
 
                     data = g_strdup_printf("%s=%s\n",Config_Variables[i].name,
                                                      *(char **)Config_Variables[i].pointer);
-                    fwrite(data,strlen(data),1,file);
-                    //g_print("# (type:%d) %s",Config_Variables[i].type,data);
+                    if (fwrite (data, strlen (data), 1, file) != 1)
+                    {
+                        Log_Print (LOG_ERROR,
+                                   _("Error while writing configuration file: %s"),
+                                   file_path);
+                        fclose (file);
+                        g_free (file_path);
+                        g_free (data);
+                        return;
+                    }
                     g_free(data);
                     break;
                 }
@@ -997,17 +1009,31 @@ Save_Config_To_File (void)
 static void
 Set_Config (gchar *line)
 {
-    gchar *var_descriptor;
-    gchar *var_value;
+    const gchar *var_descriptor;
+    const gchar *var_value;
     gint ConfigVarListLen;
     gint i;
 
+    g_return_if_fail (line != NULL);
+
     if (*line=='\n' || *line=='#') return;
 
     /* Cut string */
-    var_descriptor = strtok(line,"=");
-    var_value      = strtok(NULL,"=");
-    //g_print("\nstr1:'%s',\t str2:'%s'",var_descriptor,var_value);
+    var_descriptor = strtok (line, "=");
+
+    if (!var_descriptor)
+    {
+        g_message ("Invalid configuration file line ‘%s’", line);
+        return;
+    }
+
+    var_value = strtok (NULL, "=");
+
+    if (!var_value)
+    {
+        g_message ("No value for configuration key ‘%s’", var_descriptor);
+        return;
+    }
 
     ConfigVarListLen = sizeof(Config_Variables)/sizeof(tConfigVariable);
     for (i=0; i<ConfigVarListLen; i++)
@@ -1166,7 +1192,6 @@ Save_List_Store_To_File (const gchar *filename, GtkListStore *liststore, gint co
 {
     gchar *file_path = NULL;
     FILE *file;
-    gchar *data = NULL;
     gchar *text;
     GtkTreeIter iter;
 
@@ -1185,15 +1210,27 @@ Save_List_Store_To_File (const gchar *filename, GtkListStore *liststore, gint co
     {
         do
         {
-            gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, colnum, &text, -1);
-            data = g_strdup_printf("%s\n",text);
-            g_free(text);
+            GString *data;
 
-            if (data)
+            gtk_tree_model_get (GTK_TREE_MODEL (liststore), &iter, colnum,
+                                &text, -1);
+            data = g_string_new (text);
+            g_free (text);
+            g_string_append_c (data, '\n');
+
+            if (*data->str != '\n')
             {
-                fwrite(data,strlen(data),1,file);
-                g_free(data);
+                if (fwrite (data->str, data->len, 1, file) != 1)
+                {
+                    Log_Print (LOG_ERROR, _("Error while writing list file: %s"),
+                               file_path);
+                    fclose (file);
+                    g_string_free (data, TRUE);
+                    g_free (file_path);
+                    return;
+                }
             }
+            g_string_free (data, TRUE);
         } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(liststore), &iter));
         fclose(file);
     }
@@ -1210,7 +1247,6 @@ Populate_List_Store_From_File (const gchar *filename, GtkListStore *liststore, g
     gchar *file_path = NULL;
     FILE *file;
     gchar buffer[MAX_STRING_LEN];
-    GtkTreeIter iter;
     gboolean entries_set = FALSE;
 
     /* The file to write */
@@ -1240,8 +1276,8 @@ Populate_List_Store_From_File (const gchar *filename, GtkListStore *liststore, g
 
             if (data && g_utf8_strlen(data, -1) > 0)
             {
-                gtk_list_store_append(liststore, &iter);
-                gtk_list_store_set(liststore, &iter, text_column, data, -1);
+                gtk_list_store_insert_with_values (liststore, NULL, G_MAXINT,
+                                                   text_column, data, -1);
                 entries_set = TRUE;
             }
             g_free(data);
@@ -1256,9 +1292,11 @@ Populate_List_Store_From_File (const gchar *filename, GtkListStore *liststore, g
 /*
  * Functions for writing and reading list of 'Fill Tag' masks
  */
-void Load_Scan_Tag_Masks_List (GtkListStore *liststore, gint colnum, gchar **fallback)
+void
+Load_Scan_Tag_Masks_List (GtkListStore *liststore, gint colnum,
+                          const gchar * const *fallback)
 {
-    gint i = 0;
+    gsize i = 0;
     GtkTreeIter iter;
 
     if (!Populate_List_Store_From_File(SCAN_TAG_MASKS_FILE, liststore, colnum))
@@ -1268,8 +1306,8 @@ void Load_Scan_Tag_Masks_List (GtkListStore *liststore, gint colnum, gchar **fal
 
         while(fallback[i])
         {
-            gtk_list_store_append(liststore, &iter);
-            gtk_list_store_set(liststore, &iter, colnum, fallback[i], -1);
+            gtk_list_store_insert_with_values (liststore, &iter, G_MAXINT,
+                                               colnum, fallback[i], -1);
             i++;
         }
     }
@@ -1284,9 +1322,11 @@ void Save_Scan_Tag_Masks_List (GtkListStore *liststore, gint colnum)
 /*
  * Functions for writing and reading list of 'Rename File' masks
  */
-void Load_Rename_File_Masks_List (GtkListStore *liststore, gint colnum, gchar **fallback)
+void
+Load_Rename_File_Masks_List (GtkListStore *liststore, gint colnum,
+                             const gchar * const *fallback)
 {
-    gint i = 0;
+    gsize i = 0;
     GtkTreeIter iter;
 
     if (!Populate_List_Store_From_File(RENAME_FILE_MASKS_FILE, liststore, colnum))
@@ -1296,8 +1336,8 @@ void Load_Rename_File_Masks_List (GtkListStore *liststore, gint colnum, gchar **
 
         while(fallback[i])
         {
-            gtk_list_store_append(liststore, &iter);
-            gtk_list_store_set(liststore, &iter, colnum, fallback[i], -1);
+            gtk_list_store_insert_with_values (liststore, &iter, G_MAXINT,
+                                               colnum, fallback[i], -1);
             i++;
         }
     }
@@ -1314,7 +1354,6 @@ void Save_Rename_File_Masks_List (GtkListStore *liststore, gint colnum)
 void Load_Rename_Directory_Masks_List (GtkListStore *liststore, gint colnum, gchar **fallback)
 {
     gint i = 0;
-    GtkTreeIter iter;
 
     if (!Populate_List_Store_From_File(RENAME_DIRECTORY_MASKS_FILE, liststore, colnum))
     {
@@ -1323,8 +1362,8 @@ void Load_Rename_Directory_Masks_List (GtkListStore *liststore, gint colnum, gch
 
         while(fallback[i])
         {
-            gtk_list_store_append(liststore, &iter);
-            gtk_list_store_set(liststore, &iter, colnum, fallback[i], -1);
+            gtk_list_store_insert_with_values (liststore, NULL, G_MAXINT,
+                                               colnum, fallback[i], -1);
             i++;
         }
     }
@@ -1612,3 +1651,23 @@ Create_Easytag_Directory (void)
         return TRUE;
     }
 }
+
+static void
+set_sorting_indicator_for_column_id (gint column_id,
+                                     ET_Sorting_Type temp_sort,
+                                     GtkTreeViewColumn *column)
+{
+    GtkSortType current_sort;
+    GtkSortType sort_type;
+
+    gtk_tree_view_column_clicked (column);
+
+    current_sort = get_sort_order_for_column_id (column_id);
+
+    sort_type = temp_sort % 2 == 0 ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING;
+
+    if (sort_type != current_sort)
+    {
+        gtk_tree_view_column_clicked (column);
+    }
+}
